import * as React from 'react';
import { XIcon } from "./XIcon";
import { brushToString, stringToBrush, ensureBool, initializePropertiesFromCss, NamePatcher, isValidProp, toSpinal, getModifiedProps } from "igniteui-react-core";
import { FontInfo } from "igniteui-react-core";
import { ReactRenderer, PortalManager } from "igniteui-react-core";
export class IgrXIcon extends React.Component {
    constructor(props) {
        super(props);
        this._wrapper = null;
        this._implementation = null;
        this.__p = null;
        this._hasUserValues = new Set();
        this._stylingContainer = null;
        this._stylingParent = null;
        this._inStyling = false;
        if (this._styling) {
            NamePatcher.ensureStylablePatched(Object.getPrototypeOf(this));
        }
        this._implementation = this.createImplementation();
        this._implementation.externalObject = this;
        this._getMainRef = this._getMainRef.bind(this);
        var container;
        if (document) {
            container = document.createElement("div");
        }
        var root;
        root = container;
        if (container != null) {
            root = container;
        }
        this.requestRender = this.requestRender.bind(this);
        this._portalManager = new PortalManager("iconContent", this.requestRender);
        var ren = new ReactRenderer(root, document, true, {}, this._portalManager);
        this._wrapper = ren;
        this._container = this._wrapper.rootWrapper;
        this._container.setStyleProperty("display", "inline-block");
        this._container.setStyleProperty("vertical-align", "middle");
        var icon = this.i;
        this._icon = icon;
        icon.provideContainer(ren);
        if (props) {
            this.initializeProperties();
        }
    }
    _getMainRef(ref) {
        this._elRef = ref;
    }
    render() {
        let propChildren = this.props.children;
        let children = [];
        React.Children.forEach(propChildren, (ch) => {
            children.push(React.cloneElement(ch));
        });
        this._portalManager.onRender(children);
        let div = React.createElement("div", {
            className: "ig-x-icon igr-x-icon",
            ref: this._getMainRef,
            children: children
        });
        return div;
    }
    requestRender() {
        if (this._initialized) {
            this.setState({});
        }
    }
    //private _getLabelPortal(element: DomWrapper, portalCallback: (portal: DomPortal) => void): void {
    //    this._portalManager.getPortal(element, "TemplateContent", portalCallback);
    //}
    shouldComponentUpdate(nextProps, nextState) {
        const mod = getModifiedProps(this.props, nextProps);
        for (const p of Object.keys(mod)) {
            if (isValidProp(this, p)) {
                this[p] = mod[p];
            }
        }
        return true;
    }
    initializeProperties() {
        for (const p of Object.keys(this.props)) {
            if (isValidProp(this, p)) {
                this[p] = this.props[p];
            }
        }
    }
    // supports angular themes or custom properties set in CSS
    updateStyle() {
        this._styling(this._elRef, this);
    }
    componentWillUnmount() {
        this._icon.destroy();
        this._wrapper.destroy();
    }
    componentDidMount() {
        this._elRef.appendChild(this._container.getNativeElement());
        this.initializeContent();
    }
    initializeContent() {
        this._styling(this._container.getNativeElement(), this);
        this.updateStyle();
    }
    createImplementation() {
        return new XIcon();
    }
    get i() {
        return this._implementation;
    }
    /**
 * Gets the color to use for the actual background.
*/
    get actualFill() {
        return brushToString(this.i.bu);
    }
    set actualFill(v) {
        this.i.bu = stringToBrush(v);
    }
    /**
     * Gets the color to use for the actual text color.
    */
    get actualTextColor() {
        return brushToString(this.i.by);
    }
    set actualTextColor(v) {
        this.i.by = stringToBrush(v);
    }
    /**
     * Gets the color to use for the actual background.
    */
    get actualHoverFill() {
        return brushToString(this.i.bv);
    }
    set actualHoverFill(v) {
        this.i.bv = stringToBrush(v);
    }
    /**
     * Gets the hover stroke to use for the icon.
    */
    get actualHoverStroke() {
        return brushToString(this.i.bw);
    }
    set actualHoverStroke(v) {
        this.i.bw = stringToBrush(v);
    }
    /**
     * Gets the stroke to use for the icon.
    */
    get actualStroke() {
        return brushToString(this.i.bx);
    }
    set actualStroke(v) {
        this.i.bx = stringToBrush(v);
    }
    /**
     * Gets the hover stroke to use for the icon.
    */
    get actualHoverStrokeThickness() {
        return this.i.s;
    }
    set actualHoverStrokeThickness(v) {
        this.i.s = +v;
    }
    /**
     * Gets the stroke to use for the icon.
    */
    get actualStrokeThickness() {
        return this.i.t;
    }
    set actualStrokeThickness(v) {
        this.i.t = +v;
    }
    /**
     * Gets or sets the color to use for the actual background.
    */
    get fill() {
        return brushToString(this.i.fill);
    }
    set fill(v) {
        this.i.fill = stringToBrush(v);
    }
    /**
     * Gets or sets the color to use for the text.
    */
    get textColor() {
        return brushToString(this.i.b8);
    }
    set textColor(v) {
        this.i.b8 = stringToBrush(v);
    }
    /**
     * Gets or sets the color to use for the text.
    */
    get svgPath() {
        return this.i.svgPath;
    }
    set svgPath(v) {
        this.i.svgPath = v;
    }
    /**
     * Gets or sets the color to use for the actual background.
    */
    get hoverFill() {
        return brushToString(this.i.b4);
    }
    set hoverFill(v) {
        this.i.b4 = stringToBrush(v);
    }
    /**
     * Gets or sets the hover stroke to use for the icon.
    */
    get hoverStroke() {
        return brushToString(this.i.b5);
    }
    set hoverStroke(v) {
        this.i.b5 = stringToBrush(v);
    }
    /**
     * Gets or sets the stroke to use for the icon.
    */
    get stroke() {
        return brushToString(this.i.b7);
    }
    set stroke(v) {
        this.i.b7 = stringToBrush(v);
    }
    /**
     * Gets or sets the hover stroke to use for the icon.
    */
    get hoverStrokeThickness() {
        return this.i.w;
    }
    set hoverStrokeThickness(v) {
        this.i.w = +v;
    }
    /**
     * Gets or sets the stroke to use for the icon.
    */
    get strokeThickness() {
        return this.i.y;
    }
    set strokeThickness(v) {
        this.i.y = +v;
    }
    /**
     * Gets or sets the use for the button.
    */
    get textStyle() {
        if (this.i.e == null) {
            return null;
        }
        return this.i.e.fontString;
    }
    set textStyle(v) {
        let fi = new FontInfo();
        fi.fontString = v;
        this.i.e = fi;
    }
    /**
     * Gets or sets the color to use for the hovered text of the button regardless of type.
    */
    get hoverTextColor() {
        return brushToString(this.i.b6);
    }
    set hoverTextColor(v) {
        this.i.b6 = stringToBrush(v);
    }
    /**
     * Gets or sets the id to use for the checkbox.
    */
    get id() {
        return this.i.av;
    }
    set id(v) {
        this.i.av = v;
    }
    /**
     * Gets or sets TabIndex to use for the checkbox.
    */
    get tabIndex() {
        return this.i.aa;
    }
    set tabIndex(v) {
        this.i.aa = +v;
    }
    /**
     * Gets or sets the value of the aria-label attribute.
    */
    get ariaLabel() {
        return this.i.an;
    }
    set ariaLabel(v) {
        this.i.an = v;
    }
    /**
     * Gets or sets whether the icon is hovered.
    */
    get isHover() {
        return this.i.l;
    }
    set isHover(v) {
        this.i.l = ensureBool(v);
    }
    /**
     * Gets or sets whether the checkbox is disabled.
    */
    get disabled() {
        return this.i.disabled;
    }
    set disabled(v) {
        this.i.disabled = ensureBool(v);
    }
    get opacity() {
        return this.i.x;
    }
    set opacity(v) {
        this.i.x = +v;
    }
    findByName(name) {
        return null;
    }
    get hasUserValues() {
        return this._hasUserValues;
    }
    __m(propertyName) {
        if (!this._inStyling) {
            this._hasUserValues.add(propertyName);
        }
    }
    _styling(container, component, parent) {
        if (this._inStyling) {
            return;
        }
        this._inStyling = true;
        this._stylingContainer = container;
        this._stylingParent = component;
        let genericPrefix = "";
        let typeName = this.i.$type.name;
        if (typeName.indexOf("Xam") === 0) {
            typeName = typeName.substring(3);
        }
        genericPrefix = toSpinal("XIcon");
        let additionalPrefixes = [];
        let prefix = toSpinal(typeName);
        additionalPrefixes.push(prefix + "-");
        let b = this.i.$type.baseType;
        while (b && b.name != "Object" &&
            b.name != "Base" &&
            b.name != "Control" &&
            b.Name != "DependencyObject" &&
            b.Name != "FrameworkElement") {
            typeName = b.name;
            if (typeName.indexOf("Xam") === 0) {
                typeName = typeName.substring(3);
            }
            let basePrefix = toSpinal(typeName);
            additionalPrefixes.push(basePrefix + "-");
            b = b.baseType;
        }
        if (parent) {
            let parentTypeName = parent.i.$type.name;
            if (parentTypeName.indexOf("Xam") === 0) {
                parentTypeName = parentTypeName.substring(3);
            }
            let parentPrefix = toSpinal(parentTypeName);
            additionalPrefixes.push(parentPrefix + "-" + genericPrefix + "-");
            additionalPrefixes.push(parentPrefix + "-" + prefix + "-");
        }
        initializePropertiesFromCss(container, this, genericPrefix + "-", this.hasUserValues, false, additionalPrefixes);
        if (this._otherStyling) {
            this._otherStyling(container, component, parent);
        }
        this._inStyling = false;
    }
    onDetachedFromUI() {
        this.i.onDetachedFromUI();
    }
    onAttachedToUI() {
        this.i.onAttachedToUI();
    }
    /**
     * Exports visual information about the current state of the grid.
    
    */
    exportVisualModel() {
        let iv = this.i.ac();
        return (iv);
    }
    /**
     * Returns a serialized copy of the exported visual model
    
    */
    exportSerializedVisualModel() {
        let iv = this.i.ap();
        return (iv);
    }
}
